#ifndef WLPDSTM_TIMESTAMP_H_
#define WLPDSTM_TIMESTAMP_H_

#include "../constants.h"
#include "atomic.h"

namespace wlpdstm {

	class GlobalTimestamp {
		public:
			GlobalTimestamp() : ts(MINIMUM_TS + 1) {  }

			unsigned readCurrentTs() const {
				return ts;
			}

			unsigned readCurrentTsAcquire() const {
				return (unsigned)atomic_load_acquire(&ts);
			}

			unsigned getNextTsFull() {
				return fetch_and_inc_full(&ts);
			}

			unsigned getNextTsAcquire() {
				return fetch_and_inc_acquire(&ts);
			}

			unsigned getNextTs() {
				return fetch_and_inc_no_barrier(&ts);
			}

			void restart() {
				ts = MINIMUM_TS + 1;
			}

		private:
			unsigned ts;

			char padding[CACHE_LINE_SIZE_BYTES - sizeof(unsigned)];
	};
}

#endif // WLPDSTM_TIMESTAMP_H_
