#ifndef WLPDSTM_CONSTANTS_H_
#define WLPDSTM_CONSTANTS_H_

#define CACHE_LINE_SIZE_BYTES 64
#define CACHE_LINE_ALIGNED __attribute__((aligned(CACHE_LINE_SIZE_BYTES)))

namespace wlpdstm {

	// this constant is useful for defining various data structures
	const unsigned MAX_THREADS = 32;

	const unsigned CACHE_LINE_SIZE = 16;

	// the smallest timestamp used
	const unsigned MINIMUM_TS = 0;

	// TODO: infer this somehow for better portability
	const unsigned ADDRESS_SPACE_SIZE = 32;
	const unsigned WORD_LOG_SIZE = 22;

	const unsigned LOCK_RESERVED_BITS = 2;

	// -1 to get the maximum number
	const unsigned MAXIMUM_TS = (1 << (ADDRESS_SPACE_SIZE - LOCK_RESERVED_BITS)) - 1 - MAX_THREADS;

	// top level transaction
	const int TX_TOP_LEVEL = 0;
	const int TX_NO_LEVEL = (TX_TOP_LEVEL - 1);

	enum OperationType {
		NO_OP = 0,
		READ_OP = 1,
		WRITE_OP = 2,
		DELETE_OP = 3
	};

	inline const char *getOperationName(OperationType optype) {
		const char *names[] = {
			"noop",
			"read",
			"write",
			"delete"
		};

		return names[optype];
	}
}

#define LSB 1u

//#define MINIMUM_VERSION ((void *)LSB)
#define MINIMUM_VERSION ((void *)0)

#define WRITE_LOCK_CLEAR ((void *)0)

#define READ_LOCK_SET ((void *)LSB)

#define NO_LEXICAL_TX -1

#endif
