/** 
 * @author Aleksandar Dragojevic aleksandar.dragojevic@epfl.ch
 */

#include "wlpdstm.h"

#include "tid.h"
#include "memory.h"
#include "transaction.h"
#include "constants.h"

void wlpdstm::global_init() {
	CurrentTransaction::GlobalInit();
}

void wlpdstm::thread_init() {
	CurrentTransaction::ThreadInit();
}

void wlpdstm::start_tx() {
	CurrentTransaction::TxStart();
}

void wlpdstm::start_tx(int lexical_tx_id) {
	CurrentTransaction::TxStart(lexical_tx_id);
}

#ifdef USE_LONG_JUMP
LONG_JMP_BUF *wlpdstm::get_long_jmp_buf() {
	return CurrentTransaction::GetLongJmpBuf();
}
#endif /* USE_LONG_JMP */

void wlpdstm::commit_tx() {
	CurrentTransaction::TxCommit();
}

void wlpdstm::abort_tx() {
	CurrentTransaction::TxAbort();
}

void wlpdstm::restart_tx() {
	CurrentTransaction::TxRestart();
}

void wlpdstm::write_word(void *address, void *value) {
	CurrentTransaction::WriteWord(address, value);
}

#ifdef SUPPORT_LOCAL_WRITES
void wlpdstm::write_word_local(void *address, void *value) {
	CurrentTransaction::WriteWordLocal(address, value);
}
#endif /* SUPPORT_LOCAL_WRITES */

void *wlpdstm::read_word(void *address) {
	return CurrentTransaction::ReadWord(address);
}

void wlpdstm::tx_free(void *ptr, size_t size) {
	CurrentTransaction::TxFree(ptr, size);
}

void *wlpdstm::tx_malloc(size_t size) {
	return CurrentTransaction::TxMalloc(size);
}

wlpdstm::tx_desc *wlpdstm::get_tx_desc() {
	return (tx_desc *)CurrentTransaction::Get();
}

void wlpdstm::start_tx(tx_desc *tx) {
	((Transaction *)tx)->TxStart();
}

#ifdef USE_LONG_JUMP
LONG_JMP_BUF *wlpdstm::get_long_jmp_buf(tx_desc *tx) {
	return &((Transaction *)tx)->start_buf;
}
#endif /* USE_LONG_JMP */

void wlpdstm::start_tx(tx_desc *tx, int lexical_tx_id) {
	((Transaction *)tx)->TxStart(lexical_tx_id);
}

void wlpdstm::commit_tx(tx_desc *tx) {
	((Transaction *)tx)->TxCommit();
}

void wlpdstm::restart_tx(tx_desc *tx) {
	((Transaction *)tx)->TxRestart();
}

void wlpdstm::abort_tx(tx_desc *tx) {
	((Transaction *)tx)->TxAbort();
}

void *wlpdstm::read_word(tx_desc *tx, void *address) {
	return ((Transaction *)tx)->ReadWord(address);
}

void wlpdstm::write_word(tx_desc *tx, void *address, void *value) {
	((Transaction *)tx)->WriteWord(address, value);
}

#ifdef SUPPORT_LOCAL_WRITES
void wlpdstm::write_word_local(tx_desc *tx, void *address, void *value) {
	((Transaction *)tx)->WriteWordLocal(address, value);
}
#endif /* SUPPORT_LOCAL_WRITES */

void wlpdstm::tx_free(tx_desc *tx, void *ptr, size_t size) {
	((Transaction *)tx)->TxFree(ptr, size);
}

void *wlpdstm::tx_malloc(tx_desc *tx, size_t size) {
	return ((Transaction *)tx)->TxMalloc(size);
}

void wlpdstm::print_stats() {
	Transaction::PrintStatistics();
}

void *wlpdstm::s_malloc(size_t size) {
	return wlpdstm::MemoryManager::Malloc(size);
}

void wlpdstm::s_free(void *ptr) {
	wlpdstm::MemoryManager::Free(ptr);
}
