/**
 * @author Aleksandar Dragojevic aleksandar.dragojevic@epfl.ch
 */

#ifndef WLPDSTM_WLPDSTM_H_
#define WLPDSTM_WLPDSTM_H_

#include <cstdlib>

#ifdef USE_LONG_JUMP
#include "common/jmp.h"
#endif /* USE_LONG_JUMP */

namespace wlpdstm {

	typedef void *tx_desc;

	// initialization
	void global_init();

	void thread_init();

	// start/end tx
	void start_tx();
	void start_tx(int lexical_tx_id);

#ifdef USE_LONG_JUMP
	LONG_JMP_BUF *get_long_jmp_buf();
#endif /* USE_LONG_JMP */

	void commit_tx();

	void abort_tx();

	void restart_tx();

	// read/write word
	void *read_word(void *address);

	void write_word(void *address, void *value);

#ifdef SUPPORT_LOCAL_WRITES
	void write_word_local(void *address, void *value);
#endif /* SUPPORT_LOCAL_WRITES */

	// memory management
	void *tx_malloc(size_t size);

	void tx_free(void *ptr, size_t size);

	///////////////////////////////////////////////////////
	// a separate set of methods if current tx is cached //
	///////////////////////////////////////////////////////

	tx_desc *get_tx_desc();

	// start/end tx
	void start_tx(tx_desc *tx);
	void start_tx(tx_desc *tx, int lexical_tx_id);

#ifdef USE_LONG_JUMP
	LONG_JMP_BUF *get_long_jmp_buf(tx_desc *tx);
#endif /* USE_LONG_JMP */

	void commit_tx(tx_desc *tx);

	void abort_tx(tx_desc *tx);

	void restart_tx(tx_desc *tx);

	// read/write word
	void *read_word(tx_desc *tx, void *address);

	void write_word(tx_desc *tx, void *address, void *value);

#ifdef SUPPORT_LOCAL_WRITES
	void write_word_local(tx_desc *tx, void *address, void *value);
#endif /* SUPPORT_LOCAL_WRITES */

	// memory management
	void *tx_malloc(tx_desc *tx, size_t size);

	void tx_free(tx_desc *tx, void *ptr, size_t size);

	void print_stats();

	// use for non-tx code to be able to simply switch
	// the whole memory management scheme
	void *s_malloc(size_t size);
	void s_free(void *ptr);
}

#endif // WLPDSTM_WLPDSTM_H_
